==> mcrash-1.7.tar.gz: mcrash/Linux26/clean_net.c <==
/*
 * Copyright (C) 2000-2003 ASANO Masahiro
 */

#include "def.h"

#define __KERNEL__
#include <linux/net.h>
#include <net/sock.h>
#include <net/tcp.h>
#include <net/udp.h>

const char *
get_address_family(af)
	int af;
{
	switch (af) {
	case AF_UNSPEC:		return "UNSPEC";
	case AF_UNIX:		return "UNIX";
	case AF_INET:		return "INET";
	case AF_AX25:		return "AX25";
	case AF_IPX:		return "IPX";
	case AF_APPLETALK:	return "APPLETALK";
	case AF_NETROM:		return "NETROM";
	case AF_BRIDGE:		return "BRIDGE";
	case AF_ATMPVC:		return "ATMPVC";
	case AF_X25:		return "X25";
	case AF_INET6:		return "INET6";
	case AF_ROSE:		return "ROSE";
	case AF_DECnet:		return "DECnet";
	case AF_NETBEUI:	return "NETBEUI";
	case AF_SECURITY:	return "SECURITY";
	case AF_KEY:		return "KEY";
	case AF_NETLINK:	return "NETLINK";
	case AF_PACKET:		return "PACKET";
	case AF_ASH:		return "ASH";
	case AF_ECONET:		return "ECONET";
	case AF_ATMSVC:		return "ATMSVC";
	case AF_SNA:		return "SNA";
	case AF_IRDA:		return "IRDA";
	case AF_PPPOX:		return "PPPOX";
	case AF_WANPIPE:	return "WANPIPE";
#ifdef AF_LLC
	case AF_LLC:		return "LLC";
#endif
#ifdef AF_BLUETOOTH
	case AF_BLUETOOTH:	return "BLUETOOTH";
#endif
	default:		return "?";
	}
}

const char *
get_socket_type(typ)
	int typ;
{
	switch (typ) {
	case SOCK_STREAM:	return "STREAM";
	case SOCK_DGRAM:	return "DGRAM";
	case SOCK_RAW:		return "RAW";
	case SOCK_RDM:		return "RDM";
	case SOCK_SEQPACKET:	return "SEQPACKET";
	case SOCK_PACKET:	return "PACKET";
	default:		return "?";
	}
}

addr_t
print_socket(addr)
	addr_t addr;
{
	addr_t waddr;
	struct socket sock;
	static const struct bitname sflags[] = {
		{ 1<<SOCK_ASYNC_NOSPACE,	"async_nospace" },
		{ 1<<SOCK_ASYNC_WAITDATA,	"async_waitdata" },
		{ 1<<SOCK_NOSPACE,		"nospace" },
		{ 0, NULL }
	};

	memread(addr, sizeof(sock), &sock, "socket");
	mprintf("addr:        " FPTR "\n", addr);

	mprintf("state:       ");
	switch (sock.state) {
	case SS_FREE:		mprintf("FREE");	break;
	case SS_UNCONNECTED:	mprintf("UNCONNECTED");	break;
	case SS_CONNECTING:	mprintf("CONNECTING");	break;
	case SS_CONNECTED:	mprintf("CONNECTED");	break;
	case SS_DISCONNECTING:	mprintf("DISCONNECTING"); break;
	default:		mprintf("?");		break;
	}
	mprintf("\n");
	mprintf("flags:      "); mprintbit(sflags, sock.flags); mprintf("\n");
	mprintf("ops:         " FPTR "\n", sock.ops);
	mprintf("fasync_list: " FPTR "\n", sock.fasync_list);
	mprintf("file:        " FPTR "\n", sock.file);
	mprintf("sk:          " FPTR "\n", sock.sk);
	mprintf("wait:       ");
	waddr = (addr_t)sock.wait.task_list.next;
	while (waddr && waddr != addr + OFFSET(struct socket, wait.task_list)) {
		mprintf(" " FPTR, waddr);
		memread(waddr + OFFSET(struct list_head, next), sizeof(waddr), &waddr, "wait_queue_head");
	}
	mprintf("\n");
	mprintf("type:        %s\n", get_socket_type(sock.type));
	mprintf("passcread:   %x\n", sock.passcred);

	return 0;
}

void
prhead_sock()
{
#if LINUXVER==24
	mprintf(SPTR" FAMILY CNT   TYPE   DESTINATION         LOCAL   R Q   W Q "SPTR"\n", "ADDR", "SOCKET");
#else
	mprintf(SPTR" FAMILY CNT   TYPE   R Q   W Q "SPTR"\n", "ADDR", "SOCKET");
#endif
}

void
prhead_sk_buff()
{
	mprintf("  "SPTR" "SPTR" "SPTR" "SPTR" "SPTR" "SPTR"  LEN DLEN "SPTR"\n",
		"ADDR", "DEV", "TRANSPOR", "NETWORK", "LINK", "DST", "HEAD");
}

addr_t
print_sk_buff(addr)
	addr_t addr;
{
	struct sk_buff skb;

	memread(addr, sizeof(skb), &skb, "sk_buff");
	mprintf("  " FPTR, addr);
	mprintf(" " FPTR, skb.dev);
	mprintf(" " FPTR, skb.h.th);
	mprintf(" " FPTR, skb.nh.iph);
	mprintf(" " FPTR, skb.mac.ethernet);
	mprintf(" " FPTR, skb.dst);
	mprintf(" %4x %4x", skb.len, skb.data_len);
	mprintf(" " FPTR, skb.head);
	mprintf("\n");
	return (addr_t) skb.next;
}

addr_t
print_sock(addr, fflag)
	addr_t addr;
	int fflag;
{
	struct sock sk;

	memread(addr, sizeof(sk), &sk, "sock");
	mprintf(FPTR, addr);
	mprintf(" %6s", get_address_family(sk.sk_family));
	mprintf(" %3x", ATOMIC_READ(sk.sk_refcnt));
	mprintf(" %6s", get_socket_type(sk.sk_type));
#if LINUXVER==24
	mprintf(" %08x:%04x %08x:%04x", m_ntohl(sk.sk_daddr), m_ntohs(sk.sk_dport), m_ntohl(sk.sk_rcv_saddr), sk.sk_num);
#endif
	mprintf(" %3x %1x", ATOMIC_READ(sk.sk_rmem_alloc), sk.sk_receive_queue.qlen);
	mprintf(" %3x %1x", ATOMIC_READ(sk.sk_wmem_alloc), sk.sk_write_queue.qlen);
#if 0
	mprintf(" " FPTR, sk.sk_pair);
#endif
	mprintf(" " FPTR, sk.sk_socket);
	mprintf("\n");

	if (fflag) {
		addr_t skb;

		if (sk.sk_receive_queue.qlen) {
			prhead_sk_buff();
		}
		skb = (addr_t)sk.sk_receive_queue.next;
		while (skb && skb != addr + OFFSET(struct sock, sk_receive_queue)) {
			skb = print_sk_buff(skb);
		}

		if (sk.sk_write_queue.qlen) {
			prhead_sk_buff();
		}
		skb = (addr_t)sk.sk_write_queue.next;
		while (skb && skb != addr + OFFSET(struct sock, sk_write_queue)) {
			skb = print_sk_buff(skb);
		}
	}
	return (addr_t) sk.sk_node.next;	/*XXX*/
}

addr_t
print_listed_sock(fflag, addr, udp_hash_addr, unix_socket_table_addr)
	int fflag;
	addr_t addr;
	addr_t udp_hash_addr;
	addr_t unix_socket_table_addr;
{
	struct tcp_hashinfo tcp;
	int i;

	memread(addr, sizeof(tcp), &tcp, "tcp_hashinfo");

	mprintf("TCP ESTABLISHED\n");
	prhead_sock();
	for (i = 0; i < tcp.__tcp_ehash_size; i++) {
		struct tcp_ehash_bucket ebucket;
#ifdef DEBUG
		mprintf("[" FPTR "]\n", (addr_t)(tcp.__tcp_ehash + i));
#endif /*DEBUG*/
		memread((addr_t)(tcp.__tcp_ehash + i), sizeof(ebucket), &ebucket, "tcp_ehash_bucket");
		addr = (addr_t)ebucket.chain.first;	/*XXX*/
		while (addr) {
			addr = print_sock(addr, fflag);
		}
	}

	mprintf("TCP LISTENING\n");
	prhead_sock();
	for (i = 0; i < TCP_LHTABLE_SIZE; i++) {
		addr = (addr_t) tcp.__tcp_listening_hash[i].first;	/*XXX*/
		while (addr) {
			addr = print_sock(addr - OFFSET(struct sock, sk_node), fflag);
		}
	}

	mprintf("UDP\n");
	prhead_sock();
	for (i = 0; i < UDP_HTABLE_SIZE; i++) {
		memread(udp_hash_addr + sizeof(addr_t) * i, sizeof(addr), &addr, "udp_hash");
		while (addr) {
			addr = print_sock(addr - OFFSET(struct sock, sk_node), fflag);
		}
	}

	mprintf("UNIX\n");
	prhead_sock();
	for (i = 0; i < 256 + 1; i++) {
		memread(unix_socket_table_addr + sizeof(addr_t) * i, sizeof(addr), &addr, "unix_socket_table");
		while (addr) {
			addr = print_sock(addr - OFFSET(struct sock, sk_node), fflag);
		}
	}

	return 0;
}

==> mcrash-1.13.tar.gz: mcrash/Linux26/clean_net.c <==
/*
 * Copyright (C) 2000-2003 ASANO Masahiro
 */

#include "def.h"

#define __KERNEL__
#include <linux/net.h>
#include <net/sock.h>
#include <net/tcp.h>
#include <net/udp.h>

#include "flags_socket.h"

const char *
get_address_family(af)
	int af;
{
	switch (af) {
	case AF_UNSPEC:		return "UNSPEC";
	case AF_UNIX:		return "UNIX";
	case AF_INET:		return "INET";
	case AF_AX25:		return "AX25";
	case AF_IPX:		return "IPX";
	case AF_APPLETALK:	return "APPLETALK";
	case AF_NETROM:		return "NETROM";
	case AF_BRIDGE:		return "BRIDGE";
	case AF_ATMPVC:		return "ATMPVC";
	case AF_X25:		return "X25";
	case AF_INET6:		return "INET6";
	case AF_ROSE:		return "ROSE";
	case AF_DECnet:		return "DECnet";
	case AF_NETBEUI:	return "NETBEUI";
	case AF_SECURITY:	return "SECURITY";
	case AF_KEY:		return "KEY";
	case AF_NETLINK:	return "NETLINK";
	case AF_PACKET:		return "PACKET";
	case AF_ASH:		return "ASH";
	case AF_ECONET:		return "ECONET";
	case AF_ATMSVC:		return "ATMSVC";
	case AF_SNA:		return "SNA";
	case AF_IRDA:		return "IRDA";
	case AF_PPPOX:		return "PPPOX";
	case AF_WANPIPE:	return "WANPIPE";
#ifdef AF_LLC
	case AF_LLC:		return "LLC";
#endif
#ifdef AF_BLUETOOTH
	case AF_BLUETOOTH:	return "BLUETOOTH";
#endif
	default:		return "?";
	}
}

const char *
get_socket_type(typ)
	int typ;
{
	switch (typ) {
	case SOCK_STREAM:	return "STREAM";
	case SOCK_DGRAM:	return "DGRAM";
	case SOCK_RAW:		return "RAW";
	case SOCK_RDM:		return "RDM";
	case SOCK_SEQPACKET:	return "SEQPACKET";
	case SOCK_PACKET:	return "PACKET";
	default:		return "?";
	}
}

const char *
get_sock_state(state)
	int state;
{
	switch (state) {
	case TCP_ESTABLISHED:	return "ESTABLISH";
	case TCP_SYN_SENT:	return "SYN_SENT";
	case TCP_SYN_RECV:	return "SYN_RECV";
	case TCP_FIN_WAIT1:	return "FIN_WAIT1";
	case TCP_FIN_WAIT2:	return "FIN_WAIT2";
	case TCP_TIME_WAIT:	return "TIME_WAIT";
	case TCP_CLOSE:		return "CLOSE";
	case TCP_CLOSE_WAIT:	return "CLOSEWAIT";
	case TCP_LAST_ACK:	return "LAST_ACK";
	case TCP_LISTEN:	return "LISTEN";
	case TCP_CLOSING:	return "CLOSING";
	default:		return "?";
	}
}

addr_t
print_socket(addr)
	addr_t addr;
{
	addr_t waddr;
	struct socket sock;
	static const struct bitname sflags[] = {
		{ 1<<SOCK_ASYNC_NOSPACE,	"async_nospace" },
		{ 1<<SOCK_ASYNC_WAITDATA,	"async_waitdata" },
		{ 1<<SOCK_NOSPACE,		"nospace" },
		{ 0, NULL }
	};

	memread(addr, sizeof(sock), &sock, "socket");
	mprintf("addr:        " FPTR "\n", addr);

	mprintf("state:       ");
	switch (sock.state) {
	case SS_FREE:		mprintf("FREE");	break;
	case SS_UNCONNECTED:	mprintf("UNCONNECTED");	break;
	case SS_CONNECTING:	mprintf("CONNECTING");	break;
	case SS_CONNECTED:	mprintf("CONNECTED");	break;
	case SS_DISCONNECTING:	mprintf("DISCONNECTING"); break;
	default:		mprintf("?");		break;
	}
	mprintf("\n");
	mprintf("flags:      "); mprintbit(sflags, sock.flags); mprintf("\n");
	mprintf("ops:         " FPTR "\n", sock.ops);
	mprintf("fasync_list: " FPTR "\n", sock.fasync_list);
	mprintf("file:        " FPTR "\n", sock.file);
	mprintf("sk:          " FPTR "\n", sock.sk);
	mprintf("wait:       ");
	waddr = (addr_t)sock.wait.task_list.next;
	while (waddr && waddr != addr + OFFSET(struct socket, wait.task_list)) {
		mprintf(" " FPTR, waddr);
		memread(waddr + OFFSET(struct list_head, next), sizeof(waddr), &waddr, "wait_queue_head");
	}
	mprintf("\n");
	mprintf("type:        %s\n", get_socket_type(sock.type));
	mprintf("passcread:   %x\n", sock.passcred);

	return 0;
}

void
prhead_sock()
{
	mprintf(SPTR" FAMILY     STATE CNT   TYPE   R Q   W Q "SPTR" FLAGS\n",
		"ADDR", "SOCKET");
}

void
prhead_sk_buff()
{
	mprintf("  "SPTR" "SPTR" "SPTR" "SPTR" "SPTR" "SPTR"  LEN DLEN "SPTR"\n",
		"ADDR", "DEV", "TRANSPOR", "NETWORK", "LINK", "DST", "HEAD");
}

addr_t
print_sk_buff(addr)
	addr_t addr;
{
	struct sk_buff skb;

	memread(addr, sizeof(skb), &skb, "sk_buff");
	mprintf("  " FPTR, addr);
	mprintf(" " FPTR, skb.dev);
	mprintf(" " FPTR, skb.h.th);
	mprintf(" " FPTR, skb.nh.iph);
	mprintf(" " FPTR, skb.mac.ethernet);
	mprintf(" " FPTR, skb.dst);
	mprintf(" %4x %4x", skb.len, skb.data_len);
	mprintf(" " FPTR, skb.head);
	mprintf("\n");
	return (addr_t) skb.next;
}

addr_t
print_sock(addr, fflag)
	addr_t addr;
	int fflag;
{
	struct sock sk;

	memread(addr, sizeof(sk), &sk, "sock");
	mprintf(FPTR, addr);
	mprintf(" %6s", get_address_family(sk.sk_family));
	mprintf(" %9s", get_sock_state(sk.sk_state));
	mprintf(" %3x", ATOMIC_READ(sk.sk_refcnt));
	mprintf(" %6s", get_socket_type(sk.sk_type));
	mprintf(" %3x %1x", ATOMIC_READ(sk.sk_rmem_alloc), sk.sk_receive_queue.qlen);
	mprintf(" %3x %1x", ATOMIC_READ(sk.sk_wmem_alloc), sk.sk_write_queue.qlen);
#if 0
	mprintf(" " FPTR, sk.sk_pair);
#endif
	mprintf(" " FPTR, sk.sk_socket);
	mprintbit(soflags, sk.sk_flags);
	mprintf("\n");

	if (fflag) {
		addr_t skb;

		mprintf("\treuse           %x\n", sk.sk_reuse);
		mprintf("\tbound_dev_if    %x\n", sk.sk_bound_dev_if);
		mprintf("\tzapped          %x\n", sk.sk_zapped);
		mprintf("\tshutdown        %x\n", sk.sk_shutdown);
		mprintf("\tuse_write_queue %x\n", sk.sk_use_write_queue);
		mprintf("\tuserlocks       %x\n", sk.sk_userlocks);
		mprintf("\trcvbuf          %x\n", sk.sk_rcvbuf);
		mprintf("\tsleep           " FPTR "\n", sk.sk_sleep);
		mprintf("\tdst_cache       " FPTR "\n", sk.sk_dst_cache);
		mprintf("\tlingertime      %lx\n", sk.sk_lingertime);
		mprintf("\tpair            " FPTR "\n", sk.sk_pair);
		mprintf("\tbacklog         " FPTR "  " FPTR "\n",
			sk.sk_backlog.head, sk.sk_backlog.tail);
		mprintf("\tprot            " FPTR "\n", sk.sk_prot);
		mprintf("\terr             %x  %x\n",
			sk.sk_err, sk.sk_err_soft);
		mprintf("\tpriority        %x\n", sk.sk_priority);
		mprintf("\tlocalroute      %x\n", sk.sk_localroute);
		mprintf("\trcvlowat        %x\n", sk.sk_rcvlowat);
		mprintf("\trcv/sndtimeo    %lx  %lx\n",
			sk.sk_rcvtimeo, sk.sk_sndtimeo);
		mprintf("\tfilter          " FPTR "\n", sk.sk_filter);
		mprintf("\tprotinfo        " FPTR "\n", sk.sk_protinfo);
		mprintf("\tslab            " FPTR "\n", sk.sk_slab);
		mprintf("\tuser_data       " FPTR "\n", sk.sk_user_data);
		mprintf("\towner           " FPTR "\n", sk.sk_owner);
		mprintf("\t" FPTR " " FPTR " " FPTR " " FPTR " " FPTR " " FPTR "\n",
			sk.sk_state_change, sk.sk_data_ready,
			sk.sk_write_space,  sk.sk_error_report,
			sk.sk_backlog_rcv,  sk.sk_destruct);

		if (sk.sk_receive_queue.qlen) {
			prhead_sk_buff();
		}
		skb = (addr_t)sk.sk_receive_queue.next;
		while (skb && skb != addr + OFFSET(struct sock, sk_receive_queue)) {
			skb = print_sk_buff(skb);
		}

		if (sk.sk_write_queue.qlen) {
			prhead_sk_buff();
		}
		skb = (addr_t)sk.sk_write_queue.next;
		while (skb && skb != addr + OFFSET(struct sock, sk_write_queue)) {
			skb = print_sk_buff(skb);
		}
	}
	return (addr_t) sk.sk_node.next;	/*XXX*/
}

addr_t
print_listed_sock(fflag, addr, udp_hash_addr, unix_socket_table_addr)
	int fflag;
	addr_t addr;
	addr_t udp_hash_addr;
	addr_t unix_socket_table_addr;
{
	struct tcp_hashinfo tcp;
	int i;

	memread(addr, sizeof(tcp), &tcp, "tcp_hashinfo");

	mprintf("TCP ESTABLISHED\n");
	prhead_sock();
	for (i = 0; i < tcp.__tcp_ehash_size; i++) {
		struct tcp_ehash_bucket ebucket;
#ifdef DEBUG
		mprintf("[" FPTR "]\n", (addr_t)(tcp.__tcp_ehash + i));
#endif /*DEBUG*/
		memread((addr_t)(tcp.__tcp_ehash + i), sizeof(ebucket), &ebucket, "tcp_ehash_bucket");
		addr = (addr_t)ebucket.chain.first;	/*XXX*/
		while (addr) {
			addr = print_sock(addr, fflag);
		}
	}

	mprintf("TCP LISTENING\n");
	prhead_sock();
	for (i = 0; i < TCP_LHTABLE_SIZE; i++) {
		addr = (addr_t) tcp.__tcp_listening_hash[i].first;	/*XXX*/
		while (addr) {
			addr = print_sock(addr - OFFSET(struct sock, sk_node), fflag);
		}
	}

	mprintf("UDP\n");
	prhead_sock();
	for (i = 0; i < UDP_HTABLE_SIZE; i++) {
		memread(udp_hash_addr + sizeof(addr_t) * i, sizeof(addr), &addr, "udp_hash");
		while (addr) {
			addr = print_sock(addr - OFFSET(struct sock, sk_node), fflag);
		}
	}

	mprintf("UNIX\n");
	prhead_sock();
	for (i = 0; i < 256 + 1; i++) {
		memread(unix_socket_table_addr + sizeof(addr_t) * i, sizeof(addr), &addr, "unix_socket_table");
		while (addr) {
			addr = print_sock(addr - OFFSET(struct sock, sk_node), fflag);
		}
	}

	return 0;
}
